/*
New demosystem by Nuclear / Mindlapse
Copyright (C) 2011-2013  John Tsiombikas <nuclear@member.fsf.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DSYS2_H_
#define DSYS2_H_

#include <stdio.h>

struct dsys_demo;
struct dsys_event;

typedef float demotime_t;

enum dsys_evtype {
	DSYS_SINGLE,
	DSYS_PERIODIC
};

#ifdef __cplusplus
extern "C" {
#endif

struct dsys_demo *dsys_open(const char *fname);
struct dsys_demo *dsys_open_stream(FILE *fp);
void dsys_close(struct dsys_demo *demo);

void dsys_update(struct dsys_demo *demo, demotime_t tm);


void dsys_start(struct dsys_demo *demo);
void dsys_stop(struct dsys_demo *demo);
int dsys_is_running(struct dsys_demo *demo);


demotime_t dsys_duration(struct dsys_demo *demo);
demotime_t dsys_time(struct dsys_demo *demo);
float dsys_progress(struct dsys_demo *demo);

/* seek without continuity */
void dsys_seek(struct dsys_demo *demo, demotime_t tm);
void dsys_seek_norm(struct dsys_demo *demo, float t);

/* seek by accelerating time */
void dsys_warp(struct dsys_demo *demo, demotime_t tm);
void dsys_warp_norm(struct dsys_demo *demo, float t);


/* events */
struct dsys_event *dsys_event(struct dsys_demo *demo, const char *name);

void dsys_set_event_eval(struct dsys_event *ev, float (*eval)(struct dsys_event*, demotime_t));

enum dsys_evtype dsys_event_type(struct dsys_event *ev);
float dsys_event_value(struct dsys_event *ev);

int dsys_event_callback(struct dsys_event *ev, void (*func)(struct dsys_event*, void*), void *cls);
int dsys_event_link(struct dsys_event *ev, float *link);

/* event evaluators */
float dsys_eval_step(struct dsys_event *ev, demotime_t t);
float dsys_eval_lerp(struct dsys_event *ev, demotime_t t);
float dsys_eval_sigmoid(struct dsys_event *ev, demotime_t t);

/* time conversion */
demotime_t dsys_sec_to_dtime(float sec);
demotime_t dsys_msec_to_dtime(unsigned long msec);

float dsys_dtime_to_sec(demotime_t tm);
unsigned long dsys_dtime_to_msec(demotime_t tm);

#ifdef __cplusplus
}
#endif


#endif	/* DSYS2_H_ */
